@@ -331,13 +331,13 @@ gnome_desktop_item_new (void)
 	/* These are guaranteed to be set */
 	gnome_desktop_item_set_string (retval,
 				       GNOME_DESKTOP_ITEM_NAME,
-				       _("No name"));
+				       "No name");
 	gnome_desktop_item_set_string (retval,
 				       GNOME_DESKTOP_ITEM_ENCODING,
-				       _("UTF-8"));
+				       "UTF-8");
 	gnome_desktop_item_set_string (retval,
 				       GNOME_DESKTOP_ITEM_VERSION,
-				       _("1.0"));
+				       "1.0");
 
 	return retval;
 }
@@ -392,6 +392,11 @@ gnome_desktop_item_copy (const GnomeDesktopItem *item)
 	retval->location = g_strdup (item->location);
 	retval->mtime = item->mtime;
 
+	/* Languages */
+	retval->languages = g_list_copy (item->languages);
+	for (li = retval->languages; li != NULL; li = li->next)
+		li->data = g_strdup (li->data);	
+
 	/* Keys */
 	retval->keys = g_list_copy (item->keys);
 	for (li = retval->keys; li != NULL; li = li->next)
@@ -551,6 +556,37 @@ gnome_desktop_item_new_from_uri (const char *uri,
 
 		return NULL;
 	}
+	
+	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE &&
+	    info->type != GNOME_VFS_FILE_TYPE_REGULAR &&
+	    info->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
+		g_set_error (error,
+			     /* FIXME: better errors */
+			     GNOME_DESKTOP_ITEM_ERROR,
+			     GNOME_DESKTOP_ITEM_ERROR_INVALID_TYPE,
+			     _("File '%s' is not a regular file or directory."),
+			     uri);
+
+		gnome_vfs_file_info_unref (info);
+
+		return NULL;
+	}	    	
+	
+	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_TYPE &&
+	    info->type == GNOME_VFS_FILE_TYPE_REGULAR &&
+	    info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE &&
+	    strcmp (info->mime_type, "application/x-gnome-app-info") != 0) {
+		g_set_error (error,
+			     /* FIXME: better errors */
+			     GNOME_DESKTOP_ITEM_ERROR,
+			     GNOME_DESKTOP_ITEM_ERROR_INVALID_TYPE,
+			     _("File '%s' has invalid MIME type: %s"),
+			     uri, info->mime_type);
+
+		gnome_vfs_file_info_unref (info);
+
+		return NULL;
+	}		
 
 	if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME)
 		mtime = info->mtime;
@@ -565,7 +601,12 @@ gnome_desktop_item_new_from_uri (const char *uri,
 					     GNOME_VFS_FILE_INFO_DEFAULT) != GNOME_VFS_OK) {
 			gnome_vfs_file_info_unref (info);
 			g_free (subfn);
-			return make_fake_directory (uri);
+
+			if (flags & GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS) {
+				return NULL;
+			} else {
+				return make_fake_directory (uri);
+			}
 		}
 
 		if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME)
@@ -2033,10 +2074,13 @@ find_kde_directory (void)
  * Returns: A newly allocated string
  */
 char *
-gnome_desktop_item_find_icon (const char *icon,
+gnome_desktop_item_find_icon (GnomeIconLoader *icon_loader,
+			      const char *icon,
 			      int desired_size,
 			      int flags)
 {
+	char *full = NULL;
+
 	if (icon == NULL || strcmp(icon,"") == 0) {
 		return NULL;
 	} else if (g_path_is_absolute (icon)) {
@@ -2046,7 +2090,36 @@ gnome_desktop_item_find_icon (const char *icon,
 			return NULL;
 		}
 	} else {
-		char *full;
+		char *icon_no_extension;
+		char *p;
+
+		if (icon_loader == NULL) {
+			icon_loader = gnome_icon_loader_new ();
+		} else {
+			g_object_ref (icon_loader);
+		}
+
+		
+		icon_no_extension = g_strdup (icon);
+		p = strrchr (icon_no_extension, '.');
+		if (p &&
+		    (strcmp (p, ".png") == 0 ||
+		     strcmp (p, ".xpm") == 0 ||
+		     strcmp (p, ".svg") == 0)) {
+		    *p = 0;
+		}
+
+		full = gnome_icon_loader_lookup_icon (icon_loader,
+						      icon_no_extension,
+						      desired_size,
+						      NULL);
+		
+		g_object_unref (icon_loader);
+		
+		g_free (icon_no_extension);
+	}
+
+	if (full == NULL) { /* Fall back on old Gnome/KDE code */
 		GSList *kde_dirs = NULL;
 		GSList *li;
 		const char *exts[] = { ".png", ".xpm", NULL };
@@ -2095,8 +2168,9 @@ gnome_desktop_item_find_icon (const char *icon,
 
 		g_slist_free (kde_dirs);
 
-		return full;
 	}
+	    return full;
+	    
 }
 
 /**
@@ -2110,7 +2184,8 @@ gnome_desktop_item_find_icon (const char *icon,
  * Returns: A newly allocated string
  */
 char *
-gnome_desktop_item_get_icon (const GnomeDesktopItem *item)
+gnome_desktop_item_get_icon (const GnomeDesktopItem *item,
+			     GnomeIconLoader *icon_loader)
 {
 	/* maybe this function should be deprecated in favour of find icon
 	 * -George */
@@ -2121,7 +2196,7 @@ gnome_desktop_item_get_icon (const GnomeDesktopItem *item)
 
 	icon = gnome_desktop_item_get_string (item, GNOME_DESKTOP_ITEM_ICON);
 
-	return gnome_desktop_item_find_icon (icon,
+	return gnome_desktop_item_find_icon (icon_loader, icon,
 					     48 /* desired_size */,
 					     0 /* flags */);
 }
