@@ -20,6 +20,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <glib.h>
+#include <gconf/gconf-client.h>
 
 #include "gnome-icon-loader.h"
 #include "gnome-theme-parser.h"
@@ -47,16 +48,23 @@ typedef enum
 
 struct _GnomeIconLoaderPrivate
 {
+  gboolean custom_theme;
   char *current_theme;
   char **search_path;
   int search_path_len;
 
+  gboolean allow_svg;
+  
   gboolean themes_valid;
   /* A list of all the themes needed to look up icons.
    * In search order, without duplicates
    */
   GList *themes;
   GHashTable *unthemed_icons;
+
+
+  /* GConf data: */
+  guint theme_changed_id;
 };
 
 typedef struct
@@ -102,7 +110,11 @@ static void   theme_subdir_load            (GnomeIconLoader      *loader,
 					    IconTheme            *theme,
 					    GnomeThemeFile       *theme_file,
 					    char                 *subdir);
-static void   icon_data_destroy            (GnomeIconData             *data);
+static void   icon_data_destroy            (GnomeIconData        *data);
+static void   blow_themes                  (GnomeIconLoaderPrivate *priv);
+
+
+static guint		 signal_changed = 0;
 
 
 GType
@@ -143,17 +155,100 @@ gnome_icon_loader_class_init (GnomeIconLoaderClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = gnome_icon_loader_finalize;
+
+  signal_changed = g_signal_new ("changed",
+				 G_TYPE_FROM_CLASS (klass),
+				 G_SIGNAL_RUN_LAST,
+				 G_STRUCT_OFFSET (GnomeIconLoaderClass, changed),
+				 NULL, NULL,
+				 g_cclosure_marshal_VOID__VOID,
+				 G_TYPE_NONE, 0);
 }
 
+static void
+theme_changed (GConfClient* client,
+	       guint cnxn_id,
+	       GConfEntry *entry,
+	       gpointer user_data)
+{
+  GnomeIconLoader *loader = user_data;
+  GnomeIconLoaderPrivate *priv = loader->priv;
+  const char *str;
+  
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_STRING)
+    return;
+
+  str = gconf_value_get_string (entry->value);
+
+  g_free (priv->current_theme);
+  priv->current_theme = g_strdup (str);
+
+  blow_themes (priv);
+  g_signal_emit (G_OBJECT (loader), signal_changed, 0);
+}
+
+static void
+setup_gconf_handler (GnomeIconLoader *loader)
+{
+  GnomeIconLoaderPrivate *priv;
+  GConfClient *client;
+  char *theme;
+  
+  priv = loader->priv;
+
+  client = gconf_client_get_default ();
+
+  gconf_client_add_dir (client,
+			"/desktop/gnome/interface",
+			GCONF_CLIENT_PRELOAD_NONE,
+			NULL);
+
+  g_assert (priv->theme_changed_id == 0);
+  
+  priv->theme_changed_id = gconf_client_notify_add (client,
+						    "/desktop/gnome/interface/icon_theme",
+						    theme_changed,
+						    loader, NULL, NULL);
+
+  theme = gconf_client_get_string (client,
+				   "/desktop/gnome/interface/icon_theme",
+				   NULL);
+  
+  if (theme)
+    {
+      g_free (priv->current_theme);
+      priv->current_theme = theme;
+    }
+}
+
+static void
+remove_gconf_handler (GnomeIconLoader *loader)
+{
+  GnomeIconLoaderPrivate *priv;
+  GConfClient *client;
+  char *theme;
+  
+  priv = loader->priv;
+
+  g_assert (priv->theme_changed_id != 0);
+  
+  gconf_client_notify_remove (gconf_client_get_default (),
+			      priv->theme_changed_id);
+  priv->theme_changed_id = 0;
+}
+
+
 static void
 gnome_icon_loader_init (GnomeIconLoader *loader)
 {
   GnomeIconLoaderPrivate *priv;
 
-  priv = g_new (GnomeIconLoaderPrivate, 1);
+  priv = g_new0 (GnomeIconLoaderPrivate, 1);
   
   loader->priv = priv;
 
+  priv->custom_theme = FALSE;
   priv->current_theme = g_strdup ("default");
   priv->search_path = g_new (char *, 3);
   
@@ -168,6 +263,10 @@ gnome_icon_loader_init (GnomeIconLoader *loader)
   priv->themes_valid = FALSE;
   priv->themes = NULL;
   priv->unthemed_icons = NULL;
+
+  priv->allow_svg = FALSE;
+  
+  setup_gconf_handler (loader);
 }
 
 static void
@@ -195,17 +294,42 @@ gnome_icon_loader_finalize (GObject *object)
   priv = loader->priv;
 
   g_free (priv->current_theme);
+  priv->current_theme = NULL;
 
   for (i=0; i < priv->search_path_len; i++)
     g_free (priv->search_path[i]);
 
   g_free (priv->search_path);
+  priv->search_path = NULL;
+
+  if (priv->theme_changed_id)
+    {
+      gconf_client_notify_remove (gconf_client_get_default (),
+				  priv->theme_changed_id);
+      priv->theme_changed_id = 0;
+    }
 
   blow_themes (priv);
 
   g_free (priv);
 }
 
+void
+gnome_icon_loader_set_allow_svg (GnomeIconLoader      *loader,
+				 gboolean              allow_svg)
+{
+  allow_svg = allow_svg != FALSE;
+
+  if (allow_svg == loader->priv->allow_svg)
+    return;
+  
+  loader->priv->allow_svg = allow_svg;
+  
+  blow_themes (loader->priv);
+  
+  g_signal_emit (G_OBJECT (loader), signal_changed, 0);
+}
+
 void
 gnome_icon_loader_set_search_path (GnomeIconLoader *loader,
 				   const char *path[],
@@ -226,6 +350,7 @@ gnome_icon_loader_set_search_path (GnomeIconLoader *loader,
     priv->search_path[i] = g_strdup (path[i]);
 
   blow_themes (priv);
+  g_signal_emit (G_OBJECT (loader), signal_changed, 0);
 }
 
 
@@ -259,6 +384,7 @@ gnome_icon_loader_append_search_path (GnomeIconLoader      *loader,
   priv->search_path[priv->search_path_len-1] = g_strdup (path);
 
   blow_themes (priv);
+  g_signal_emit (G_OBJECT (loader), signal_changed, 0);
 }
 
 void
@@ -281,19 +407,31 @@ gnome_icon_loader_prepend_search_path (GnomeIconLoader      *loader,
   priv->search_path[0] = g_strdup (path);
 
   blow_themes (priv);
+  g_signal_emit (G_OBJECT (loader), signal_changed, 0);
 }
 
 void
-gnome_icon_loader_set_current_theme (GnomeIconLoader *loader,
-				     const char *theme_name)
+gnome_icon_loader_set_custom_theme (GnomeIconLoader *loader,
+				    const char *theme_name)
 {
   GnomeIconLoaderPrivate *priv;
 
   priv = loader->priv;
   g_free (priv->current_theme);
-  priv->current_theme = g_strdup (theme_name);
+  if (theme_name != NULL)
+    {
+      priv->current_theme = g_strdup (theme_name);
+      priv->custom_theme = TRUE;
+      remove_gconf_handler (loader);
+    }
+  else
+    {
+      priv->custom_theme = FALSE;
+      setup_gconf_handler (loader);
+    }
 
   blow_themes (priv);
+  g_signal_emit (G_OBJECT (loader), signal_changed, 0);
 }
 
 static void
@@ -302,11 +440,13 @@ insert_theme (GnomeIconLoader *loader, const char *theme_name)
   int i;
   GList *l;
   char **dirs;
+  char **themes;
   GnomeIconLoaderPrivate *priv;
   IconTheme *theme;
   char *path;
   char *contents;
   char *directories;
+  char *inherits;
   GnomeThemeFile *theme_file;
   
   priv = loader->priv;
@@ -384,11 +524,26 @@ insert_theme (GnomeIconLoader *loader, const char *theme_name)
   theme->dirs = g_list_reverse (theme->dirs);
 
   g_free (directories);
-  
-  gnome_theme_file_free (theme_file);
-  
+
   /* Prepend the finished theme */
   priv->themes = g_list_prepend (priv->themes, theme);
+
+  if (gnome_theme_file_get_string (theme_file,
+				   "Icon Theme",
+				   "Inherits",
+				   &inherits))
+    {
+      themes = g_strsplit (inherits, ",", 0);
+
+      for (i = 0; themes[i] != NULL; i++)
+	insert_theme (loader, themes[i]);
+      
+      g_strfreev (themes);
+
+      g_free (inherits);
+    }
+
+  gnome_theme_file_free (theme_file);
 }
 
 static gboolean
@@ -423,7 +578,8 @@ load_themes (GnomeIconLoader *loader)
   priv = loader->priv;
 
   insert_theme (loader, priv->current_theme);
-
+  priv->themes = g_list_reverse (priv->themes);
+  
   priv->unthemed_icons = g_hash_table_new_full (g_str_hash, g_str_equal,
 						g_free, g_free);
 
@@ -438,7 +594,7 @@ load_themes (GnomeIconLoader *loader)
       while ((file = g_dir_read_name (gdir)))
 	{
 	  if (my_g_str_has_suffix (file, ".png") ||
-	      my_g_str_has_suffix (file, ".svg") ||
+	      (priv->allow_svg && my_g_str_has_suffix (file, ".svg")) ||
 	      my_g_str_has_suffix (file, ".xpm"))
 	    {
 	      abs_file = g_build_filename (dir, file, NULL);
@@ -854,7 +1010,7 @@ load_icon_data (IconThemeDir *dir, const char *path, const char *name)
 }
 
 static void
-scan_directory (IconThemeDir *dir, char *full_dir)
+scan_directory (IconThemeDir *dir, char *full_dir, gboolean allow_svg)
 {
   GDir *gdir;
   const char *name;
@@ -889,7 +1045,7 @@ scan_directory (IconThemeDir *dir, char *full_dir)
       
       if (my_g_str_has_suffix (name, ".png")) 
 	suffix = ICON_SUFFIX_PNG;
-      else if (my_g_str_has_suffix (name, ".svg")) 
+      else if (allow_svg && my_g_str_has_suffix (name, ".svg")) 
 	suffix = ICON_SUFFIX_SVG;
       else if (my_g_str_has_suffix (name, ".xpm"))
 	suffix = ICON_SUFFIX_XPM;
@@ -992,7 +1148,7 @@ theme_subdir_load (GnomeIconLoader *loader,
 	  dir->dir = full_dir;
 	  dir->icon_data = NULL;
 	  
-	  scan_directory (dir, full_dir);
+	  scan_directory (dir, full_dir, loader->priv->allow_svg);
 
 	  theme->dirs = g_list_append (theme->dirs, dir);
 	}
